import 'dart:math';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;
import '../constants/constant.dart';

class ChartTitle {
  pw.Row sessionheading() {
    return pw.Row(children: [
      pw.Container(
          width: 350,
          alignment: pw.Alignment.topLeft,
          padding: const pw.EdgeInsets.all(5),
          child: pw.Text(Constant.metadata,
              style: pw.TextStyle(
                color: PdfColors.white,
                fontSize: 10,
                fontWeight: pw.FontWeight.bold,
              ))),
      pw.VerticalDivider(color: PdfColors.white),
      pw.Container(
          alignment: pw.Alignment.topLeft,
          padding: const pw.EdgeInsets.all(5),
          child: pw.Text(Constant.sessiondetails,
              style: pw.TextStyle(
                color: PdfColors.white,
                fontSize: 10,
                fontWeight: pw.FontWeight.bold,
              ))),
    ]);
  }

  pw.Stack graphtitle(String title, PdfColor baseColor) {
    return pw.Stack(children: [
      pw.Divider(color: baseColor, thickness: 1.5),
      pw.Align(
        alignment: pw.Alignment.topLeft,
        child: pw.Container(
          width: 120,
          height: 20,
          margin: const pw.EdgeInsets.only(top: 8),
          padding: const pw.EdgeInsets.all(3),
          color: Constant.titlebackground,
          child: pw.Text(
            "$title",
            textAlign: (pw.TextAlign.center),
            style: pw.TextStyle(
                fontSize: 10,
                fontWeight: pw.FontWeight.bold,
                color: PdfColors.white),
          ),
        ),
      ),
    ]);
  }

  pw.Container leftlabel() {
    return pw.Container(
      alignment: pw.Alignment.center,
      margin: const pw.EdgeInsets.only(right: 5, top: 10),
      child: pw.Transform.rotateBox(
          angle: pi / 2,
          child: pw.Text(Constant.leftlabel,
              style: const pw.TextStyle(fontSize: 8))),
    );
  }

  pw.LineDataSet linegraph(
      PdfColor baseColor, List<int> yAxis, List<int> xAxis) {
    return pw.LineDataSet(
      drawSurface: true,
      isCurved: true,
      drawPoints: true,
      pointSize: 2.5,
      color: baseColor,
      data: List<pw.LineChartValue>.generate(
        yAxis.length,
        (i) {
          final v = yAxis[i] as num;
          return pw.LineChartValue(xAxis[i].toDouble(), v.toDouble());
        },
      ),
    );
  }

  pw.CartesianGrid chartaxis(List<int> xAxis, String filterXaxis(int time)) {
    return pw.CartesianGrid(
      xAxis: pw.FixedAxis(xAxis,
          format: (axisvalue) => filterXaxis(axisvalue as int),
          divisions: false),
      yAxis: pw.FixedAxis(
        [0, 50, 100, 150],
        format: (axisvalue) => '$axisvalue\%',
        divisions: false,
      ),
    );
  }
}
